﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Xml.Serialization;

namespace MMDSScv.Konfiguration
{
    class Settings
    {

        /// <summary>
        /// Objekt mit der aktuellen Konfiguration
        /// </summary>
        public Config currConfig;

        /// <summary>
        /// Pfad zur aktuellen Konfiguration
        /// </summary>
        private string configFilePath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\MMDSScv\config.xml";

        /// <summary>
        /// Liefert den Pfad zur Konfigurationsdatei
        /// </summary>
        /// <returns>Pfad zur Konfigurationsdatei</returns>
        public string GetConfigPath()
        {
            return configFilePath;
        }

        /// <summary>
        /// Erstellt ein neues "Settings" Objekt - als Pfad zur aktuellen Konfiguration wird die Datei "config.xml" im Anwedungsverzeichnis verwendet
        /// </summary>
        public Settings()
        {
            //configFilePath = System.Windows.Forms.Application.StartupPath + @"\config.xml";
            //configFilePath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\MMDSScv\config.xml";
            /*
            if (File.Exists(configFilePath))
            {
                try
                {
                    currConfig = ReadConfig(configFilePath);
                    //System.Windows.Forms.MessageBox.Show("Konfig gelesen. SettingString1: " + currConfig.settingString1);
                }
                catch (Exception ex)
                {
                    System.Windows.Forms.MessageBox.Show("Beim lesen der Konfiguration ist ein Fehler aufgetreten. Es wird die Standardkonfiguration geladen\n\n" + ex.Message
                        + "\n" + ex.InnerException);
                    currConfig = new Config();
                }
            }
            else
            {
                System.Windows.Forms.MessageBox.Show("Keine config.xml gefunden. Standardkonfiguration wird geladen.");
                currConfig = new Config();
            }*/
            SettingsStart();
        }

        /// <summary>
        /// Erstellt ein neues "Settings" Objekt - als Pfad zur aktuellen Konfiguration wird der angegebene Pfad verwendet.
        /// </summary>
        /// <param name="pathToConfig"></param>
        public Settings(string pathToConfig) : base()
        {
            configFilePath = pathToConfig;
            SettingsStart();
        }


        private void SettingsStart()
        {
            if (File.Exists(configFilePath))
            {
                try
                {
                    currConfig = ReadConfig(configFilePath);
                    //System.Windows.Forms.MessageBox.Show("Konfig gelesen. SettingString1: " + currConfig.settingString1);
                }
                catch (Exception ex)
                {
                    System.Windows.Forms.MessageBox.Show("Beim lesen der Konfiguration ist ein Fehler aufgetreten. Es wird die Standardkonfiguration geladen\n\n" + ex.Message
                        + "\n" + ex.InnerException);
                    currConfig = new Config();
                }
            }
            else
            {
                System.Windows.Forms.MessageBox.Show("Konfigurationsdatei nicht gefunden. Standardkonfiguration wird geladen. \n Erwarteter Pfad: \n"
                    + configFilePath);
                currConfig = new Config();
            }
        }

        /// <summary>
        /// Zeigt den "Einstellungen"-Dialog an. Liefert dieser "DialogResult true" zurück, wird die currConfig gleich in den angegebenen Pfad gespeichert.
        /// </summary>
        public void ShowSettingsDialog()
        {
            SettingsDialog sd = new SettingsDialog(currConfig);
            sd.ShowDialog();
            if (sd.DialogResult == System.Windows.Forms.DialogResult.OK)
            {
                SaveConfig();
            }
        }

        #region XML-File(neu)

        /// <summary>
        /// Speichert die aktuelle Konfiguration ("currConfig") in den angegebenen Dateipfad
        /// </summary>
        /// <param name="Path">Datei in die "currConfig" zu speichern ist</param>
        /// <returns>true wenn erfolgreich</returns>
        public bool SaveConfig(string Path)
        {
            XmlIO xmlio = new XmlIO(Path);
            return xmlio.SaveObjectToXmlFile(typeof(Config), currConfig);
        }

        /// <summary>
        /// Speichert die aktuelle Konfiguration ("CurrConfig") in den "configFilePath"
        /// </summary>
        /// <returns>true wenn erfolgreich</returns>
        public bool SaveConfig()
        {
            return SaveConfig(configFilePath);
        }

        /// <summary>
        /// Liefert ein "Config"-Objekt mit der in der angegebenen XML-Datei gespeicherten Konfiguration ab
        /// </summary>
        /// <param name="FilePath">Datei zur XML-Datei mit der Konfiguration</param>
        /// <returns></returns>
        public Config ReadConfig(string FilePath)
        {
            XmlIO xmlio = new XmlIO(FilePath);
            return (Config)xmlio.ReadObjectFromXmlFile(typeof(Config));
        }

        /// <summary>
        /// Speichert in das "currConfig" Ojekt die in "configFilePath" angegebene XML-Konfiguration, wenn der Lesevorgang erfolgreich war
        /// </summary>
        /// <returns>true wenn erfolgreich</returns>
        public bool ReadConfig()
        {
            bool rückgabe = false;

            Config geleseneConfig = ReadConfig(configFilePath);
            if (geleseneConfig != null)
            {
                currConfig = geleseneConfig;
                rückgabe = true;
            }
            return rückgabe;
        }

        #endregion
    }

    /// <summary>
    /// Stellt Methoden bereit, um ein Objekt in eine XML-Datei zu schreiben, oder von dort auszulesen
    /// </summary>
    class XmlIO
    {
        /// <summary>
        /// Standardkonstruktor
        /// </summary>
        public XmlIO()
        { }

        /// <summary>
        /// Konsturktor mit Übergabe des Dateipfad
        /// </summary>
        /// <param name="pathToXmlFile"></param>
        public XmlIO(string pathToXmlFile)
        {
            xmlFilePath = pathToXmlFile;
        }

        /// <summary>
        /// Pfad zur XML-Datei
        /// </summary>
        private string xmlFilePath;

        /// <summary>
        /// Speichert das angegebene Objekt als XML-Datei in den angegebenen Pfad
        /// </summary>
        /// <param name="typ">Typ des übergebenen Objekts</param>
        /// <param name="obj">Zu speicherndes Objekt</param>
        /// <param name="pathToXmlFile">Pfad zur XML-Datei</param>
        /// <returns>true wenn erfolgreich</returns>
        public bool SaveObjectToXmlFile(Type typ, object obj, string pathToXmlFile)
        {
            bool rückgabe = false;

            XmlSerializer serializer = new XmlSerializer(typ);
            //XmlSerializer serializer = new XmlSerializer(typeof(BackUpHelper.BackEnd.Settings.Config));
            if(!Directory.Exists(Path.GetDirectoryName(pathToXmlFile)))
            {
                Directory.CreateDirectory(Path.GetDirectoryName(pathToXmlFile));
            }

            FileStream filestream = new FileStream(pathToXmlFile, FileMode.Create);

            try
            {
                serializer.Serialize(filestream, obj);
                rückgabe = true;
            }
            catch (Exception ex)
            {
                throw new Exception("Fehler seralisieren/schreiben: ", ex);
            }
            finally
            {
                filestream.Close();
            }

            return rückgabe;
        }

        /// <summary>
        /// Speichert das angegebene Objekt als XML-Datei in den "xmlFilePath"
        /// </summary>
        /// <param name="typ">Typ des übergebenen Objekt</param>
        /// <param name="obj">Zu speicherndes Objekt</param>
        /// <returns>true wenn erfolgreich</returns>
        public bool SaveObjectToXmlFile(Type typ, object obj)
        {
            return SaveObjectToXmlFile(typ, obj, xmlFilePath);
        }

        /// <summary>
        /// Liest ein Objekt aus einer XML-Datei aus
        /// </summary>
        /// <param name="pathToXmlFile">Pfad zur XML-Datei</param>
        /// <returns>gelesenes Objekt</returns>
        public object ReadObjectFromXmlFile(Type typ, string pathToXmlFile)
        {
            object rückgabe = null;

            if (File.Exists(pathToXmlFile))
            {
                //XmlSerializer serializer = new XmlSerializer(typeof(object));
                XmlSerializer serializer = new XmlSerializer(typ);
                FileStream filestream = new FileStream(pathToXmlFile, FileMode.Open);

                if (File.Exists(xmlFilePath))
                {
                    try
                    {
                        rückgabe = serializer.Deserialize(filestream);
                    }
                    catch (Exception ex)
                    {
                        throw ex;
                    }
                    finally
                    {
                        filestream.Close();
                    }
                }
            }
            else
            {
                throw new Exception("Datei " + xmlFilePath + "wurde nicht gefunden.");
            }

            return rückgabe;
        }

        /// <summary>
        /// Liest ein Objekt aus der in "xmlFilePath" angegebenen XML-Datei ein
        /// </summary>
        /// <returns>gelesenes Objekt</returns>
        public object ReadObjectFromXmlFile(Type typ)
        {
            return ReadObjectFromXmlFile(typ, xmlFilePath);
        }
    }
}
