﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace MMDSScv.Konfiguration
{
    public partial class SettingsDialog : Form
    {

        /// <summary>
        /// Felder und Methoden dieses Dialoges, die nicht geändert werden sollten
        /// </summary>
        #region Dialogeigene Methoden und Felder
        private SettingsDialog()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Die akutelle Konfiguration
        /// </summary>
        private Config currConfig;

        /// <summary>
        /// Ruft den "Einstellungen"-Dialog auf.
        /// </summary>
        /// <param name="currentConfig">Die zu bearbeitende Konfiguration</param>
        internal SettingsDialog(Config currentConfig)
        {
            InitializeComponent();
            currConfig = currentConfig;
            setFields();
        }

        private void Window_Loaded(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// Bei Klick auf "OK" Felder in currKonfig sichern und Dialog schließen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOk_Click(object sender, EventArgs e)
        {
            if (System.IO.Directory.Exists(tB_PictureDir.Text))
            {
                saveFields();
                DialogResult = DialogResult.OK;
                this.Close();
            }
            else
            {
                MessageBox.Show("Verzeichnis\n\n" + tB_PictureDir.Text + "\n\nexistiert nicht! Bitte neu wählen!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Bei Klick auf "Cancel" einfach das Dialogfeld schließen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {

        }
        #endregion

        /// <summary>
        /// Hier sollten die beiden Methoden auf die hinzugefügten Felder angepasst werden.
        /// </summary>
        #region zuÄnderndeFelder

        /// <summary>
        /// Wird beim laden des Fensters aufgerufen und soll Inhalte der "currConfig" in die Felder schreiben
        /// </summary>
        private void setFields()
        {
            tB_PictureDir.Text = currConfig.DirectoryPath;
            cB_ShowSubDirs.Checked = currConfig.SearchSubDirs;
            numeric_Interval.Value = currConfig.TimerInterval / 1000;
            cB_ChangeAlternate.Checked = currConfig.ChangeAlternate;
            cB_ShowFileName.Checked = currConfig.PictureInfoBoxConfig.ShowFileName;
            cB_ShowTitle.Checked = currConfig.PictureInfoBoxConfig.ShowTitle;
            cB_ShowAutor.Checked = currConfig.PictureInfoBoxConfig.ShowPhotographer;

            switch (currConfig.PictureInfoBoxConfig.LabelConfig.BoxPosition)
            {
                case SpecialLabels.SelfPositionatingLabelConfig.Position.top_left:
                    {
                        radBtnTop_Left.Checked = true;
                        break;
                    }
                case SpecialLabels.SelfPositionatingLabelConfig.Position.top_right:
                    {
                        RadBtnTop_Right.Checked = true;
                        break;
                    }
                case SpecialLabels.SelfPositionatingLabelConfig.Position.bottom_left:
                    {
                        radBtnBottom_Left.Checked = true;
                        break;
                    }
                case SpecialLabels.SelfPositionatingLabelConfig.Position.bottom_right:
                    {
                        radBtnBottom_Right.Checked = true;
                        break;
                    }
            }
        }
        

        /// <summary>
        /// Wird beim Klick auf "OK" aufgerufen und soll die FeldInhalte in die "currConfig" sichern
        /// </summary>
        private void saveFields()
        {
            currConfig.DirectoryPath = tB_PictureDir.Text;
            currConfig.SearchSubDirs = cB_ShowSubDirs.Checked;
            currConfig.TimerInterval = (int)numeric_Interval.Value * 1000;
            currConfig.ChangeAlternate = cB_ChangeAlternate.Checked;
            currConfig.PictureInfoBoxConfig.ShowFileName = cB_ShowFileName.Checked;
            currConfig.PictureInfoBoxConfig.ShowTitle = cB_ShowTitle.Checked;
            currConfig.PictureInfoBoxConfig.ShowPhotographer = cB_ShowAutor.Checked;
            currConfig.PictureInfoBoxConfig.LabelConfig.BoxPosition = GetInfoBoxPositonFromForm();

        }

        /// <summary>
        /// Wertet die Radioboxen für die Positon des Infolabels aus und gibt sie zurück
        /// </summary>
        /// <returns></returns>
        private SpecialLabels.SelfPositionatingLabelConfig.Position GetInfoBoxPositonFromForm()
        {
            SpecialLabels.SelfPositionatingLabelConfig.Position InfoBoxPositon = SpecialLabels.SelfPositionatingLabelConfig.Position.top_left;
            if (RadBtnTop_Right.Checked)
            {
                InfoBoxPositon = SpecialLabels.SelfPositionatingLabelConfig.Position.top_right;
            }
            else if (radBtnBottom_Left.Checked)
            {
                InfoBoxPositon = SpecialLabels.SelfPositionatingLabelConfig.Position.bottom_left;
            }
            else if (radBtnBottom_Right.Checked)
            {
                InfoBoxPositon = SpecialLabels.SelfPositionatingLabelConfig.Position.bottom_right;
            }

            return InfoBoxPositon;
        }
        #endregion

        /// <summary>
        /// ButtonEventHandler um den FolderBrowserDialog für das Bilderverzeichnis anzeigen.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnShowFolderBrowserDialog_Click(object sender, EventArgs e)
        {
            ShowFolderBrowserDialog();
        }

        /// <summary>
        /// Zeigt den FolderBrowserDialog um das Bilderverzeichnis anzugeien.
        /// </summary>
        private void ShowFolderBrowserDialog()
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            //fbd.RootFolder = Environment.SpecialFolder.MyPictures;

            if (fbd.ShowDialog() == DialogResult.OK)
            {
                if(System.IO.Directory.Exists(fbd.SelectedPath))
                {
                    tB_PictureDir.Text = fbd.SelectedPath;
                }
                else
                {
                    MessageBox.Show("Verzeichnis " + fbd.SelectedPath + " existiert nicht! Bitte neu wählen!", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    ShowFolderBrowserDialog();
                }
            }
        }

        /// <summary>
        /// Eine Vorschau des Bildschirmschoners anzeigen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPreview_Click(object sender, EventArgs e)
        {
            // Diese Funktion speichert die aktuell Eingestellte Configuration in einer eigenen Konfigurationsdatei
            // und öffnet eine neue Instanz mit der erstellten Konfigurationsdatei. Die bisherige Konfiguratiosndatei
            // wird nicht verändert!

            System.IO.FileInfo fi = new System.IO.FileInfo(System.Reflection.Assembly.GetExecutingAssembly().Location); //Zuerst eigenen Anwendungspfad ermitteln
            string ApplicationPath = Application.StartupPath + "\\" + fi.Name; //Dann Programmname hinzufügen
            string PrevSettingsPath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\MMDSScv\PrevConfig.xml"; //Nun Pfad für PreviewSetting ermitteln

            Settings PrevSettings = new Settings(); //weiteres Settings-Objekt für die PreView erstellen
            PrevSettings.currConfig.DirectoryPath = tB_PictureDir.Text; //und aktuelle Einstellungen in dessen currentConfig speichern
            PrevSettings.currConfig.SearchSubDirs = cB_ShowSubDirs.Checked;
            PrevSettings.currConfig.TimerInterval = (int)numeric_Interval.Value * 1000;
            PrevSettings.currConfig.ChangeAlternate = cB_ChangeAlternate.Checked;

            PrevSettings.SaveConfig(PrevSettingsPath); //Jetzt die Konfiguration für die Vorschau speichern

            System.Diagnostics.Process.Start(ApplicationPath, "/prev \"" + PrevSettingsPath + "\"");    //Und eine neue Instanz des Programmes mit dem PreviewSetting als Parameter laden        
        }

        /// <summary>
        /// Sollte eine PreviewConfigurationsdatei vorhanden sein, wird diese gelöscht.
        /// </summary>
        private void DeletePreviewSettings()
        {
            string PrevSettingsPath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\MMDSScv\PrevConfig.xml";
            if(System.IO.File.Exists(PrevSettingsPath))
            {
                try
                {
                    System.IO.File.Delete(PrevSettingsPath);
                }
                catch
                { }
            }
        }

        /// <summary>
        /// Form wird geschlossen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SettingsDialog_FormClosed(object sender, FormClosedEventArgs e)
        {
            DeletePreviewSettings(); //PreviewKonfigurationsdatei löschen
        }
    }
}

