﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;

namespace MMDSScv.ScreenSaver
{

    public class FileManagmentConfig
    {
        /// <summary>
        /// Verzeichnispfad in dem nach Dateien gesucht werden soll
        /// </summary>
        public string FilePath = Environment.GetFolderPath(Environment.SpecialFolder.MyComputer);

        /// <summary>
        /// Nach diesen Dateiendungen wird gesucht
        /// </summary>
        public string[] AllowExt = { ".jpg", ".bmp", ".jpeg", ".JPG", ".BMP", ".JPEG" };

        /// <summary>
        /// Sollen auch Unterverzeichnisse durchsucht werden?
        /// </summary>
        public bool SearchSubDirs = true;

        public FileManagmentConfig()
        {}

        public FileManagmentConfig(string filePath, string[] allowExt, bool searchSubdirs)
        {
            FilePath = filePath;
            AllowExt = allowExt;
            SearchSubDirs = searchSubdirs;
        }

    }

    /// <summary>
    /// Diese Klasse scant ein Verzeichnis (und evlt. dessen Unterverzeichnisse) nach Dateien mit bestimmten Endungen und kann dann zufällig eine Ausspucken 
    /// </summary>
    class FileManagment
    {
        FileManagmentConfig Konfig;
        
        /// <summary>
        /// Liste mit allen gefundenen Dateien
        /// </summary>
        private List<string> FileList = new List<string>();

        /// <summary>
        /// Scant Verzeichnis (und evlt. dessen Unterverzeichnisse) nach Dateien mit bestimmten Endungen und speichert diese
        /// </summary>
        /// <param name="config">Konfigurationsobjekt</param>
        public FileManagment(FileManagmentConfig config)
        {
            Konfig = config;

            FileList = ScanDirectory(Konfig.FilePath, Konfig.AllowExt, Konfig.SearchSubDirs);
        }

        /// <summary>
        /// Liest das Verzeichnis und dessen Unterverzeichnisse ein
        /// </summary>
        /// <param name="path">Verzeichnispfad der, mit Unterverzeichnissen, gescannt werden soll</param>
        /// <param name="allowExt">Nach welchen Dateiendungen soll gesucht werden?</param>
        /// <returns>Liste mit gefundenen Dateien</returns>
        private List<string> ScanDirectory(string path, string[] allowExt)
        {
            return ScanDirectory(path, allowExt, true);
        }

        /// <summary>
        /// Verzeichnispfad, der gescannt werden soll.
        /// </summary>
        /// <param name="path">Verzeichnispfadd der gescannt werden soll</param>
        /// <param name="allowExt">Nach welchen Dateieendungen soll gesucht werden?</param>
        /// <param name="scanSubDirs">Sollen Unterverzeichnisse mitgescannt werden?</param>
        /// <returns>Liste mit gefundenen Dateien</returns>
        private List<string> ScanDirectory(string path, string[] allowExt, bool scanSubDirs)
        {
            List<string> Result = new List<string>();
            
            //erst alle Dateien Durchsuchen
            foreach (string File in Directory.GetFiles(path))
            {
                try
                {
                    if (allowExt.Contains(Path.GetExtension(File)))
                    {
                        //Wenn die Dateieendung gesucht wird - ab auf die Liste!
                        Result.Add(File);
                    }
                }
                catch { }
            }

            if (scanSubDirs) //Wenn die Unterverzeichnisse auch durchsucht werden sollen
            {
                foreach (string SubDir in Directory.GetDirectories(path))
                {
                    //jedes Unterverzeichniss durchsuchen
                    try
                    {
                        //sich selbst aufrufen und Ergebnis in die Liste schupsen
                        Result.AddRange(ScanDirectory(SubDir, allowExt));
                    }
                    catch { }
                }
            }

            return Result; //Liste übergeben
        }

        /// <summary>
        /// Für zufällige Dateien brauchen wir Zufallszahlen!
        /// </summary>
        Random Rnd = new Random();

        /// <summary>
        /// Liefert eine zufällige Datei im diesen Objekt zugeordneten Verzeichnis und evtl. dessen Unterverzeichnissen.
        /// </summary>
        /// <returns></returns>
        public string GetRandomFile()
        {
            string RandomPic = FileList[Rnd.Next(0, FileList.Count - 1)];
            if (File.Exists(RandomPic))
            {
                return RandomPic;
            }
            else
            {
                FileList.Remove(RandomPic);
                return GetRandomFile();
            }
        }

        /// <summary>
        /// Entfernt eine Datei aus der Dateiliste (löscht diese aber natürlich NICHT!)
        /// </summary>
        /// <param name="fileToRemove">Die Datei die aus der Liste entfernt werden soll</param>
        public void RemoveFilee(string fileToRemove)
        {
            FileList.Remove(fileToRemove);
        }


    }
}


