﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using MetadataExtractor.Formats.Exif;
using MetadataExtractor.Formats.Iptc;
using MetadataExtractor.Formats.Jpeg;
using MetadataExtractor;
using MMDSScv.SpecialLabels;

namespace MMDSScv.ScreenSaver
{
    public partial class ScreenSaverForm : Form
    {
        /// <summary>
        /// Der Bildschirm den dieses Forumular zugeordnet ist.
        /// </summary>
        public Screen ThisScreen = Screen.PrimaryScreen; //Vorsichtshalber mal initalisieren.

        /// <summary>
        /// Bild das beim Starten des Formulars angezeigt werden soll
        /// </summary>
        string StartPicture = "";

        /// <summary>
        /// Weil Events dieser Klasse zu abonieren in der ScreenSaverMain Klasse irgendwie nicht funzt.
        /// </summary>
        ScreenSaver.ScreenSaverMain screenSaverMain;


        /// <summary>
        /// Zeigt Informationen über das Bild an
        /// </summary>
        private SelfPositionatingOutlineLabel PictureInfoLabel;

        private Konfiguration.PictureInfoBoxConfig PictureInfoBoxConfig;

        /// <summary>
        /// From welche den Screensaver anzeigt
        /// </summary>
        /// <param name="screen">Bildschirm auf dem diese Form angezeigt werden soll</param>
        /// <param name="ssm">ScreenSaver.ScreenSaverMain ist doof und aboniert keine Events dieser Form!</param>
        /// <param name="startPicture">Bild das beim start dieser Form angezeigt werden soll</param>
        public ScreenSaverForm(Screen screen, ScreenSaver.ScreenSaverMain ssm, string startPicture, Konfiguration.PictureInfoBoxConfig pictureInfoBoxConfig)
        {
            InitializeComponent();
            ThisScreen = screen;
            StartPicture = startPicture;

            screenSaverMain = ssm;

            PictureInfoBoxConfig = pictureInfoBoxConfig;


            PictureInfoLabel = new SelfPositionatingOutlineLabel(pictureInfoBoxConfig.LabelConfig, ThisScreen)
            {
                Location = new Point(10, 10),
                Name = "PictureInfoLabel",
                Size = new Size(1000, 1000),
                BackColor = Color.Transparent,
                ForeColor = Color.White,
                OutlineForeColor = Color.Black,
                OutlineWidth = 3,
                AutoSize = true,
                Parent = pictureBox
            };
            this.Controls.Add(PictureInfoLabel);
        }

        /// <summary>
        /// Wird beim laden der Form aufgerufen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ScreenSaverForm_Load(object sender, EventArgs e)
        {
            FormBorderStyle = System.Windows.Forms.FormBorderStyle.None; // Rahmen entfernen
            this.Bounds = ThisScreen.Bounds; //Größe über kompletten Bildschirm strecken
            Cursor.Hide(); // Mauszeiger ausblenden
            TopMost = true; // Formular als oberstes Programm definieren, so wird auch die Taskleiste ausgeblendet

            pictureBox.Location = new Point(0, 0); //Eckpunkt der PictureBox in Bildschirmecke setzen...
            pictureBox.Size = ThisScreen.Bounds.Size; //..und Größe über den kompletten restlichen Bildschirm strecken

            SetPicture(StartPicture);
            PictureInfoLabel.Parent = pictureBox; //Parent muss gesetzt werden, damit die PictureBox für den Transparenten Hintergrund verwendet wird
            PictureInfoLabel.BringToFront(); //Und ganz nach vorne bringen

        }
        /// <summary>
        /// Vorherige Postion des Mauszeigers - MausMove wird beim Starten des Formulars ausgelöst
        /// </summary>
        Point PrevPos;

        /// <summary>
        /// Die Maus wurde bewegt!
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ScreenSaverForm_MouseMove(object sender, MouseEventArgs e)
        {
            // Falls vorheriger Punkt gesetzt wurde, neue Position mit alter vergleichen.
            if (!PrevPos.IsEmpty)
            {
                // wurde die Maus bewegt, Formular beenden
                if (PrevPos.X != e.X || PrevPos.Y != e.Y)
                {
                    OnUserInteraction(e);
                }
            }
            // Vorherige Position setzen (ist nur beim 1. Aufruf der Fall).
            // Close() alleine als Anweisung in dieser Funktion würde den falschen Effekt erzielen,
            // da das Ergeignis MouseMove beim Starten des Formulars immer einmal ausgelöst wird.
            PrevPos = new Point(e.X, e.Y);
        }

        /// <summary>
        /// Eine Maustaste wurde gedrückt!
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ScreenSaverForm_MouseDown(object sender, MouseEventArgs e)
        {
            OnUserInteraction(e);
        }

        /// <summary>
        /// Das Fenster fängt nicht jeden Tastendruck ein, die PictrueBox bietet nur Preview für die Tastaturtasten...
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pictureBox_PreviewKeyDown(object sender, PreviewKeyDownEventArgs e)
        {
            OnUserInteraction(e);
        }

        /// <summary>
        /// Eine Taste wurde gedrückt!
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ScreenSaverForm_KeyDown(object sender, KeyEventArgs e)
        {
            OnUserInteraction(e);
        }

        /// <summary>
        /// Der Benutzer hat ein Eingabegerät benutzt!
        /// </summary>
        public event EventHandler<EventArgs> UserInteraction;

        /// <summary>
        /// Der Benutzer hat irgendwas gemacht!
        /// </summary>
        /// <param name="e"></param>
        protected virtual void OnUserInteraction(EventArgs e)
        {
            screenSaverMain.ScreenSaverMain_UserInteraction(this, e); //Event abonieren in ScreenSaverMain funktioniert nicht ordentlich. Darum so stumpf.
            UserInteraction?.Invoke(this, e);
        }

        /// <summary>
        /// Schließt diese Form (Programm beenden)
        /// </summary>
        /// <returns></returns>
        public bool CloseThisForm()
        {
            //Threadübergreifend? => Invoke
            if (this.InvokeRequired)
            {
                return (bool)this.Invoke((Func<bool>)CloseThisForm);
            }

            this.Close();
            return true;
        }

        /// <summary>
        /// Setzt ein anderes Bild und zeigt dieses an
        /// </summary>
        /// <param name="picturePath">Pfad zum Bild</param>
        /// <returns></returns>
        public bool SetPicture(string picturePath)
        {
            if (this.InvokeRequired)
            {
                return (bool)this.Invoke((Func<string, bool>)SetPicture, picturePath);
            }
            pictureBox.Load(picturePath);
            SetPictureInfo(picturePath);
            return true;
        }

        /// <summary>
        /// Zeigt den InfoText an
        /// </summary>
        /// <param name="picturePath"></param>
        /// <returns></returns>
        private bool SetPictureInfo(string picturePath)
        {
            if (this.InvokeRequired)
            {
                return (bool)this.Invoke((Func<string, bool>)SetPictureInfo, picturePath);
            }

            PictureInfo PictureInfo = new PictureInfo(picturePath);

            StringBuilder TextPictureInfo = new StringBuilder();
            if (PictureInfoBoxConfig.ShowFileName)
            {
                TextPictureInfo.AppendLine(PictureInfo.FileName);
            }

            if (PictureInfo.Title != null && PictureInfoBoxConfig.ShowTitle)
            {
                TextPictureInfo.AppendLine("Title: " + PictureInfo.Title);
            }

            if (PictureInfo.Autor != null && PictureInfoBoxConfig.ShowPhotographer)
            {
                TextPictureInfo.AppendLine("Photographer: " + PictureInfo.Autor);
            }

            PictureInfoLabel.Text = TextPictureInfo.ToString();

            return true;
        }
    }

    /// <summary>
    /// Extrahiert aus einem Bild die gespeicherten Metadaten
    /// </summary>
    public class PictureInfo
    {

        private string _filename;
        private IList<Directory> directories;
        private IptcDirectory IptcDirectory;

        /// <summary>
        /// Extrahiert aus dem angegebenen Bild die Metadaten
        /// </summary>
        /// <param name="picturePath"></param>
        public PictureInfo(string picturePath)
        {
            System.IO.FileInfo fi = new System.IO.FileInfo(picturePath);
            _filename = fi.Name;
            if (System.IO.File.Exists(picturePath))
            {
                directories = ImageMetadataReader.ReadMetadata(picturePath);
                IptcDirectory = directories.OfType<IptcDirectory>().FirstOrDefault();
            }
        }

        /// <summary>
        /// Dateiname
        /// </summary>
        public string FileName
        {
            get { return _filename; }
        }

        /// <summary>
        /// Metadaten die dem Bildtitel entsprechend
        /// </summary>
        public string Title
        {
            get { return GetTitle(); }
        }

        /// <summary>
        /// Metadaten die dem Bildautor entsprechend
        /// </summary>
        public string Autor
        {
            get { return GetAutor(); }
        }

        /// <summary>
        /// Liefert die Metadaten die dem Bildtitel entsprechen zurück
        /// IPTC - ObjectName
        /// </summary>
        /// <returns></returns>
        private string GetTitle()
        {
            string Title = null;
            Title = IptcDirectory?.GetDescription(IptcDirectory.TagObjectName);
            return Title;
        }

        /// <summary>
        /// Liefert die Metadaten die dem Bildautor entsprechen zurück
        /// IPTC - ByLine
        /// </summary>
        /// <returns></returns>
        private string GetAutor()
        {
            string ByLine = null;
            ByLine = IptcDirectory?.GetDescription(IptcDirectory.TagByLine);
            return ByLine;
        }


    }
}
