﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace MMDSScv.ScreenSaver
{
    public class ScreenSaverMain
    {
        /* Dies ist das Hauptprogramm, es erstellt für jeden Bildschirm eine Bildschirmfüllende Screensaver Form und zeigt darin den Bildschirmschoner an.
         * 
         * Dieser Bildschirmschoner zeigt auf jedem Bildschirm ein zufälliges Bild aus einem Verzeichnis an. Es kann das Verzeichnis gewählt werden
         * ob Unterverzeichnisse auch verwendet werden sollen, die Zeit bis zum Bildwechsel und ob auf allen Bildschirmen das Bild gleichzeitig oder immer 
         * nur auf einem das Bild gewechselt werden soll.         * 
         * */

        /// <summary>
        /// Wenn true wird nur auf dem zweiten Bildschirm die Form erzeugt. ACHTUNG: nur Entwicklerzwecke! Ist kein zweiter Bildschirm vorhanden, kann es zu einem Fehler kommen!
        /// Zudem verhält sich das Programm nicht 100%ig gleich! (Warumauchimmer)
        /// </summary>
        bool test = false;

        /// <summary>
        /// Neues Settingsobjekt erzeugen, was die Konfigurationsdatei einliest.
        /// </summary>
        Konfiguration.Settings set = new Konfiguration.Settings();

        /// <summary>
        /// Aktuelle Konfiguration für den Bildschirmschoner
        /// </summary>
        MMDSScv.Konfiguration.Config Konfig;

        /// <summary>
        /// Liest und liefert die Bilddateien.
        /// </summary>
        FileManagment FileManager;

        /// <summary>
        /// Timer für den Bilderwechsel
        /// </summary>
        System.Timers.Timer timer = new System.Timers.Timer();

        /// <summary>
        /// Beinhaltet für jeden Bildschirm eine Form für den Bildschirmschoner
        /// </summary>
        private List<ScreenSaverForm> SaverForms = new List<ScreenSaverForm>();

        /// <summary>
        /// Hautprogramm des Bildschirmschoners
        /// </summary>
        public ScreenSaverMain()
        {
            Konfig = set.currConfig;
            SetParameters();
            //Für jeden Bildschirm eine Form in der Liste erzeugen
            foreach (Screen screen in Screen.AllScreens)
            {
                SaverForms.Add(new ScreenSaverForm(screen, this, FileManager.GetRandomFile(), new Konfiguration.PictureInfoBoxConfig()));
            }
        }

        /// <summary>
        /// Hauptprogramm des Bildschirmschoners mit alternativer Konfiguration (i.d.R. PreView)
        /// </summary>
        /// <param name="alternativeKonfig"></param>
        public ScreenSaverMain(Konfiguration.Config alternativeKonfig)
        {
            Konfig = alternativeKonfig;
            SetParameters();
            //Für jeden Bildschirm eine Form in der Liste erzeugen
            foreach (Screen screen in Screen.AllScreens)
            {                
                SaverForms.Add(new ScreenSaverForm(screen, this, FileManager.GetRandomFile(), alternativeKonfig.PictureInfoBoxConfig));
                //Der Form den Bildschirm zuweisen, dieses Objekt übergeben (Murks!), und schonmal ein zufälliges Bild übergebeen
            }
        }

        /// <summary>
        /// Bildschirmschoner starten
        /// </summary>
        public void Run()
        {
            ShowScreensaver();
        }

        /// <summary>
        /// Erzeugt wichtige Objekte, bzw. stellt diese ein, mit den jetzt geladenen Konfigurationsparametern
        /// </summary>
        private void SetParameters()
        {
            FileManager = new FileManagment(Konfig.FileManagerConfig);

            timer.Interval = Konfig.TimerInterval;
            timer.Elapsed += Timer_Elapsed;
        }

        /// <summary>
        /// Zeigt den Bildschirmschoner
        /// </summary>
        private void ShowScreensaver()
        {
            timer.Start();
            if (!test) //Standardmodus
            {
                //Zeigt für jedes Fenster die im Konstruktor erzeugten Formen mit den Bildschirmschonern an.
                Parallel.ForEach(SaverForms, (SaverForm) =>
                {
                        System.Windows.Forms.Application.Run(SaverForm); //Bildschirmschonerformen starten.
                                                                         //SaverForm.UserInteraction += HandleUserInteraction;
                                                                         // Aus irgendeinem verdammten Grund, abonniert der das Event nicht ordentlich

                        //SaverForm.SetPicture(FileManager.GetRandomPicture());
                    
                });
            }
            
            else if (test) //Wenn Test aktiv => Entwicklung => Screensaver nur auf sekundärem Bildschirm
            {
                Application.Run(SaverForms[1]); //Nur die Form [1] anzeigen - Achtung: nur Entwickler. Sollte ohne zweiten Bildschirm einen IndexOutOfRange bringen. Verhält sich irgendwie anders
            }
           
        }

        /// <summary>
        /// Wenn der Timer anschlägt...
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Timer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            ChangePictures(); //Bild wechseln
        }

        /// <summary>
        /// Welcher Bildschirm wurde zuletzt gewechselt?
        /// </summary>
        int LastChangedScreen = 0; // 0 = Primärer Bildschirm - wird halt dann beim ersten "Loop" als letztet gewechselt *Schulterzuck*

        /// <summary>
        /// Wechselt an allen Bildschirmen, das Bild. Je nach Einstellung überall gleichzeitig, oder durchwechselnd
        /// </summary>
        void ChangePictures()
        {
            if(!Konfig.ChangeAlternate) //wenn an allen Bildschirmen gleichzeitig das Bild gewechselt werden soll
            {               
                //einfach alle Bildschirme durchgehen
                Parallel.ForEach(SaverForms, (SaveForm) =>
                {
                    ChangePicture(SaveForm);
                });
            }

            if(Konfig.ChangeAlternate) //wenn abwechselnd gewechselt werden soll
            {
                LastChangedScreen++; //erst den nächsten Bildschirm auswählen
                if(LastChangedScreen > SaverForms.Count -1) //Obacht: habe ich überhaupt noch einen weiteren Bildschirm?
                {
                    LastChangedScreen = 0; //Wenn nicht fangen wir wieder bei 0 - Pirmärbildschirm an
                }

                ChangePicture(SaverForms[LastChangedScreen]); //und Bild wechseln! (Last ChangedScreen wurde vorher um 1 erhöht!)
            }
        }

        /// <summary>
        /// Wechselt für den püergebenen Bildschirm das Bild
        /// </summary>
        /// <param name="saverForm"></param>
        void ChangePicture(ScreenSaverForm saverForm)
        {
            string NextPicture = FileManager.GetRandomFile(); //erst ein zufälliges Bild laden
            try //Kann ja immer zu Ladefehlern beim Bild kommen, hier abfragen denn...
            {
                saverForm.SetPicture(NextPicture);
            }
            catch
            {
                //... hier kann ich einfach ein weiteres Bild probieren aber zuvor das missglückte aus dem Pool entfernen
                FileManager.RemoveFilee(NextPicture);
                ChangePicture(saverForm);
            }
        }

        /// <summary>
        /// Wenn sich irgendwas in den Fenstern tut - alle Fenster schließen - Programm beenden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void ScreenSaverMain_UserInteraction(object sender, EventArgs e)
        { 
            CloseScreensaver();
        }

        /// <summary>
        /// Schließt alle Bildschirmschonerfenster
        /// </summary>
        private void CloseScreensaver()
        {
            foreach (ScreenSaverForm SaverForm in SaverForms)
            {
                SaverForm.CloseThisForm(); //Fenster den Schließbefehl schicken!
            }
        }
    }
}
