﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace MMDSScv.SpecialLabels
{
    /// <summary>
    /// Zeichnet einen Text als Label mit Umriss
    /// </summary>
    public class OutlineLabel : Label
    {
        public OutlineLabel()
        {
            OutlineForeColor = Color.Green;
            OutlineWidth = 2;
        }
        /// <summary>
        /// Farbe des Umrisses
        /// </summary>
        public Color OutlineForeColor { get; set; }

        /// <summary>
        /// Breite des Umrisses in Pixel
        /// </summary>
        public float OutlineWidth { get; set; }

        protected override void OnPaint(PaintEventArgs e)
        {
            e.Graphics.FillRectangle(new SolidBrush(BackColor), ClientRectangle);
            using (System.Drawing.Drawing2D.GraphicsPath gp = new System.Drawing.Drawing2D.GraphicsPath())
            using (Pen outline = new Pen(OutlineForeColor, OutlineWidth)
            { LineJoin = System.Drawing.Drawing2D.LineJoin.Round })
            using (StringFormat sf = new StringFormat())
            using (Brush foreBrush = new SolidBrush(ForeColor))
            {
                gp.AddString(Text, Font.FontFamily, (int)Font.Style,
                    Font.Size, ClientRectangle, sf);
                e.Graphics.ScaleTransform(1.3f, 1.35f);
                e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
                e.Graphics.DrawPath(outline, gp);
                e.Graphics.FillPath(foreBrush, gp);
            }
        }
    }
}
