﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MMDSScv.SpecialLabels
{

    /// <summary>
    /// Configuration für eine SelfPositionatingOutlineLabel
    /// </summary>
    public class SelfPositionatingLabelConfig
    {
        /// <summary>
        /// Positionmöglichkeiten der Infobox auf dem Bildschirm
        /// </summary>
        public enum Position { top_left, top_right, bottom_left, bottom_right };

        /// <summary>
        /// Positon der Infobox auf dem Bildschirm
        /// </summary>
        public Position BoxPosition = Position.top_left;

        /// <summary>
        /// Abstand zur Bildschirmkante
        /// </summary>
        public int Padding = 10;

        /// <summary>
        /// Config mit Standardeinstellungen erstellen
        /// </summary>
        public SelfPositionatingLabelConfig()
        { }

        /// <summary>
        /// Config mit speziellen einstellungen erstellen
        /// </summary>
        /// <param name="postion">Position auf dem Bildschirm</param>
        /// <param name="screen">Bildschirm auf dem die Box angezeigt werden soll</param>
        public SelfPositionatingLabelConfig(SelfPositionatingLabelConfig.Position postion)
        {
            BoxPosition = postion;
        }
    }

    /// <summary>
    /// SelfPositionatingOutlineLabel - OutlineLabel das sich selbst auf dem Bildschirm positioniert
    /// </summary>
    public class SelfPositionatingOutlineLabel : OutlineLabel
    {
        /// <summary>
        /// Konfiguration dieser Infobox
        /// </summary>
        private SelfPositionatingLabelConfig Config;

        private Screen ThisScreen;

        /// <summary>
        /// Label das sich selbst auf dem Bildschirm positoniert
        /// </summary>
        /// <param name="config">Configuration der SelfPositionatingOutlineLabel</param>
        public SelfPositionatingOutlineLabel(SelfPositionatingLabelConfig config, Screen thisScreen)
        {
            Config = config;
            this.Layout += InfoBox_Layout;
            ThisScreen = thisScreen;
        }

        /// <summary>
        /// Event wenn die das Layout (Size) der Infobox geändert wird
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void InfoBox_Layout(object sender, LayoutEventArgs e)
        {
            SetToPosition(Config.BoxPosition);
        }

        /// <summary>
        /// Postioniert die Infobox auf dem Bildschirm in der angegebenen Postion
        /// </summary>
        /// <param name="position"></param>
        public void SetToPosition(SelfPositionatingLabelConfig.Position position)
        {
            System.Drawing.Point Position = new System.Drawing.Point(Config.Padding, Config.Padding); //Position für "top_left"

            switch (position)
            {
                case SelfPositionatingLabelConfig.Position.top_right:
                    //Es muss nur rechts/links (X) zur Positon top_left angepasst werden
                    Position.X = ThisScreen.Bounds.Width - this.Width - Config.Padding;
                    break;

                case SelfPositionatingLabelConfig.Position.bottom_left:
                    //Es muss nur oben/unten (Y) zur Postion top_left angepasst werden
                    Position.Y = ThisScreen.Bounds.Height - this.Height - Config.Padding;
                    break;


                case SelfPositionatingLabelConfig.Position.bottom_right:
                    //Sowohl rechts/links (X) wie auch oben/unten (Y) müssen angepasst werden
                    Position.X = ThisScreen.Bounds.Width - this.Width - Config.Padding;
                    Position.Y = ThisScreen.Bounds.Height - this.Height - Config.Padding;

                    break;
            }
            //Und jetzt die Postion übernehmen
            this.Location = Position;
        }
    }
}
